/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* core12_db.h was generated from the DDL file core12_db.ddl by the ddlp.
 * utility.
 */
#include "core12_db.h"

/* core12_db_dbd.h was generated from the DDL
 * file core12_db.ddl by the ddlp utility.  It is required to use the d_open_ptr API
 */
#include "core12_db_dbd.h"

/* EXAMPLE - Core for C */
/* Each time you run this example:
 *   1) The database is initialized (all existing contents removed)
 *   2) A transaction is started
 *   3) Several NA offices are added to the database
 *   4) All offices in the database are displayed (including uncommitted)
 *   5) The transaction is committed
 *   6) All offices in the database are displayed (all now committed)
 *   7) Another transaction is started
 *   8) Several EMEA offices are added to the database
 *   9) All offices in the database are displayed (including uncommitted)
 *  10) The transaction is rolled back
 *  11) All offices in the database are displayed (uncommitted EMEA offices
 *      are no longer in the database)
 *
 * The DDL (Database Definition Language) is in core12_db.ddl. The database
 * schema is embedded into the application and the database will be created 
 * the first time the application is run.  Subsequent runs will use the now
 * existing database
 *
 * The following RDM Functions are illustrated:
 *      d_opentask/d_closetask
 *      d_open_ptr/d_close
 *      d_initialize
 *      d_trbegin/d_trend/d_trabort
 *      d_fillnew
 *      d_keyfrst/d_keynext
 *      d_recread
 *      d_errorname
 *      d_errorstr
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 *
 * Each of these functions returns an integer status code, where the
 * value S_OKAY indicates a successful call.
 *
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDM call.
 *
 * The actual database will be stored in a directory named 'core12_db'
 */

int32_t display_offices(
    DB_TASK *task,
    int32_t  dbn)
{
    int32_t       rdm_status;
    struct office office_rec;

    for (rdm_status = d_keyfrst (OFFICE_NAME, task, dbn);
         rdm_status == S_OKAY;
         rdm_status = d_keynext (OFFICE_NAME, task, dbn))
    {
        /* Read and display the current person record */
        d_recread(&office_rec, task, dbn);
        printf("%s\n", office_rec.name);
    }

    /* Expect rdm_status to be S_NOTFOUND when we exit the loop */
    if(rdm_status == S_NOTFOUND)
    {
        rdm_status = S_OKAY;
    }
    return rdm_status;
}

/* =======================================================================
*/
int32_t main(void)
{
    DB_TASK       *task;
    int32_t        rdm_status;
    struct office  office_rec;

    /* create the context for the database operations */
    d_opentask(&task);

    /* Open the database for exclusive access, this will create the database
     * dictionary file (core12_db.dbd) if it does not already exist.
     */
    rdm_status = d_open_ptr("core12_db", "x", &core12_db_dbd, sizeof(core12_db_dbd), task);
    if (rdm_status != S_OKAY)
    {
        fprintf (stderr, "Exclusive open failed, error = %s: %s\n", d_errorname(rdm_status), d_errorstr(rdm_status) );
        d_closetask (task);
        return (1);
    }

    /* Initialize the databases.  Initializing will create the data files if 
     * they do not exist.  If the the data files do already exist they will be
     * removed and new empty files will be created.
     */
    d_initialize(task, CURR_DB);

    /* Start a database transaction */
    rdm_status = d_trbegin("Add NA Offices", task);
    if (rdm_status != S_OKAY)
    {
        fprintf (stderr, "Error starting a transaction, error = %s: %s\n", d_errorname(rdm_status), d_errorstr(rdm_status) );
        d_close(task);
        d_closetask (task);
        return (1);
    }

    /* Add NA offices to the database */
    strncpy (office_rec.name, "Seattle", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    strncpy (office_rec.name, "Boise", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    strncpy (office_rec.name, "San Francisco", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    strncpy (office_rec.name, "Dallas", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);
    
    /* Display all offices (includes uncommitted) */
    printf("\nAll Offices before transaction commit.\n");
    display_offices(task, CURR_DB);

    /* Commit the database transaction */
    rdm_status = d_trend(task);
    if (rdm_status != S_OKAY)
    {
        fprintf (stderr, "Error committing a transaction, error = %s: %s\n", d_errorname(rdm_status), d_errorstr(rdm_status) );
        d_close(task);
        d_closetask (task);
        return (1);
    }
    
    /* Display all offices */
    printf("\nAll Offices after transaction commit.\n");
    display_offices(task, CURR_DB);

    /* Start a database transaction */
    rdm_status = d_trbegin("Add EMEA Offices", task);
    if (rdm_status != S_OKAY)
    {
        fprintf (stderr, "Error starting a transaction, error = %s: %s\n", d_errorname(rdm_status), d_errorstr(rdm_status) );
        d_close(task);
        d_closetask (task);
        return (1);
    }

    strncpy (office_rec.name, "Paris", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    strncpy (office_rec.name, "London", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    strncpy (office_rec.name, "Dublin", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    strncpy (office_rec.name, "Zurich", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    strncpy (office_rec.name, "Madrid", SIZEOF_OFFICE_NAME);
    rdm_status = d_fillnew (OFFICE, &office_rec, task, CURR_DB);

    /* Display all offices (includes uncommitted) */
    printf("\nAll Offices before transaction rollback.\n");
    display_offices(task, CURR_DB);

    /* Rollback the database transaction */
    rdm_status = d_trabort(task);
    if (rdm_status != S_OKAY)
    {
        fprintf (stderr, "Error rolling back a transaction, error = %s: %s\n", d_errorname(rdm_status), d_errorstr(rdm_status) );
        d_close(task);
        d_closetask (task);
        return (1);
    }

    /* Display all offices */
    printf("\nAll Offices after transaction rollback.\n");
    display_offices(task, CURR_DB);
    
    /* close the database */
    d_close (task);
    d_closetask (task);
    
    printf ("\nPress <Enter> to continue . . .\n");
    getchar ();
}
